@@ -1,3 +1,6 @@
+# pylint: disable=attribute-defined-outside-init
+from __future__ import annotations
+
 import glob
 import json
 import os
@@ -25,6 +28,8 @@ from . import (
     includes,
     inlineTags,
     InputSource,
+    language,
+    line,
     lint,
     markdown,
     mdnspeclinks,
@@ -33,24 +38,29 @@ from . import (
     refs,
     retrieve,
     shorthands,
+    t,
+    testsuite,
     unsortedJunk as u,
     wpt,
 )
 
+if t.TYPE_CHECKING:
+    import widlparser
+
 
 class Spec:
     def __init__(
         self,
-        inputFilename,
-        debug=False,
-        token=None,
-        lineNumbers=False,
-        fileRequester=None,
-        testing=False,
+        inputFilename: str,
+        debug: bool = False,
+        token: t.Optional[str] = None,
+        lineNumbers: bool = False,
+        fileRequester: t.Optional[retrieve.DataFileRequester] = None,
+        testing: bool = False,
     ):
         catchArgparseBug(inputFilename)
-        self.valid = False
-        self.lineNumbers = lineNumbers
+        self.valid: bool = False
+        self.lineNumbers: bool = lineNumbers
         if lineNumbers:
             # line-numbers are too hacky, so force this to be a dry run
             constants.dryRun = True
@@ -61,50 +71,46 @@ class Spec:
                 "No input file specified, and no *.bs or *.src.html files found in current directory.\nPlease specify an input file, or use - to pipe from STDIN."
             )
             return
-        self.inputSource = InputSource.InputSource(inputFilename, chroot=constants.chroot)
-        self.transitiveDependencies = set()
-        self.debug = debug
-        self.token = token
-        self.testing = testing
+        self.inputSource: InputSource.InputSource = InputSource.inputFromName(inputFilename, chroot=constants.chroot)
+        self.transitiveDependencies: t.Set[InputSource.InputSource] = set()
+        self.debug: bool = debug
+        self.token: t.Optional[str] = token
+        self.testing: bool = testing
+        self.dataFile: retrieve.DataFileRequester
         if fileRequester is None:
             self.dataFile = retrieve.defaultRequester
         else:
             self.dataFile = fileRequester
 
-        self.md = None
-        self.mdBaseline = None
-        self.mdDocument = None
-        self.mdCommandLine = None
-        self.mdDefaults = None
-        self.mdOverridingDefaults = None
-        self.lines = []
-        self.document = None
-        self.html = None
-        self.head = None
-        self.body = None
-        self.fillContainers = None
+        self.lines: t.List[line.Line] = []
         self.valid = self.initializeState()
 
-    def initializeState(self):
-        self.normativeRefs = {}
-        self.informativeRefs = {}
+    def initializeState(self) -> bool:
+        self.normativeRefs: t.Dict[str, biblio.BiblioEntry] = {}
+        self.informativeRefs: t.Dict[str, biblio.BiblioEntry] = {}
         self.refs: refs.ReferenceManager = refs.ReferenceManager(fileRequester=self.dataFile, testing=self.testing)
-        self.externalRefsUsed = defaultdict(lambda: defaultdict(dict))
-        self.md = None
-        self.mdBaseline = metadata.MetadataManager()
-        self.mdDocument = None
-        self.mdCommandLine = metadata.MetadataManager()
-        self.mdDefaults = None
-        self.mdOverridingDefaults = None
-        self.biblios = {}
-        self.typeExpansions = {}
-        self.macros = defaultdict(lambda x: "???")
-        self.canIUse = {}
-        self.mdnSpecLinks = {}
-        self.widl = idl.getParser()
-        self.testSuites = json.loads(self.dataFile.fetch("test-suites.json", str=True))
-        self.languages = json.loads(self.dataFile.fetch("languages.json", str=True))
-        self.extraStyles = defaultdict(str)
+        self.externalRefsUsed: t.Any = defaultdict(lambda: defaultdict(dict))
+
+        self.md: metadata.MetadataManager
+        self.mdBaseline: t.Optional[metadata.MetadataManager] = metadata.MetadataManager()
+        self.mdDocument: t.Optional[metadata.MetadataManager] = None
+        self.mdCommandLine: t.Optional[metadata.MetadataManager] = metadata.MetadataManager()
+        self.mdDefaults: t.Optional[metadata.MetadataManager] = None
+        self.mdOverridingDefaults: t.Optional[metadata.MetadataManager] = None
+
+        self.typeExpansions: t.Dict[str, str] = {}
+
+        defaultMacro: t.Callable[[], str] = lambda: "???"
+        self.macros: t.DefaultDict[str, str] = defaultdict(defaultMacro)
+
+        self.canIUse: t.Optional[caniuse.CanIUseManager] = None
+        self.mdnSpecLinks: t.Any = {}
+        self.widl: widlparser.Parser = idl.getParser()
+
+        self.testSuites: t.Dict[str, testsuite.TestSuite] = fetchTestSuites(self.dataFile)
+        self.languages: t.Dict[str, language.Language] = fetchLanguages(self.dataFile)
+
+        self.extraStyles: t.DefaultDict[str, str] = defaultdict(str)
         self.extraStyles["style-colors"] = styleColors
         self.extraStyles["style-darkmode"] = styleDarkMode
         self.extraStyles["style-md-lists"] = styleMdLists
@@ -112,7 +118,7 @@ class Spec:
         self.extraStyles["style-selflinks"] = styleSelflinks
         self.extraStyles["style-counters"] = styleCounters
         self.extraStyles["style-issues"] = styleIssues
-        self.extraScripts = defaultdict(str)
+        self.extraScripts: t.DefaultDict[str, str] = defaultdict(str)
 
         try:
             inputContent = self.inputSource.read()
@@ -131,12 +137,13 @@ class Spec:
     def recordDependencies(self, *inputSources):
         self.transitiveDependencies.update(inputSources)
 
-    def preprocess(self):
+    def preprocess(self) -> Spec:
         self.transitiveDependencies.clear()
         self.assembleDocument()
         self.processDocument()
+        return self
 
-    def assembleDocument(self):
+    def assembleDocument(self) -> Spec:
         # Textual hacks
         u.stripBOM(self)
         if self.lineNumbers:
@@ -191,22 +198,23 @@ class Spec:
         self.refs.setSpecData(self.md)
 
         # Convert to a single string of html now, for convenience.
-        self.html = "".join(line.text for line in self.lines)
+        self.html = "".join(x.text for x in self.lines)
         boilerplate.addHeaderFooter(self)
         self.html = self.fixText(self.html)
 
         # Build the document
         self.document = h.parseDocument(self.html)
-        self.head = h.find("head", self)
-        self.body = h.find("body", self)
+        self.head = t.cast("t.ElementT", h.find("head", self))
+        self.body = t.cast("t.ElementT", h.find("body", self))
         u.correctFrontMatter(self)
         includes.processInclusions(self)
         metadata.parseDoc(self)
+        return self
 
-    def processDocument(self):
+    def processDocument(self) -> Spec:
         # Fill in and clean up a bunch of data
         conditional.processConditionals(self)
-        self.fillContainers = u.locateFillContainers(self)
+        self.fillContainers: t.FillContainersT = u.locateFillContainers(self)
         lint.exampleIDs(self)
         wpt.processWptElements(self)
 
@@ -286,8 +294,8 @@ class Spec:
                 h.removeNode(el)
             # Make sure the W3C stylesheet is after all other styles.
             for el in h.findAll("link", self):
-                if el.get("href").startswith("https://www.w3.org/StyleSheets/TR"):
-                    h.appendChild(h.find("head", self), el)
+                if el.get("href", "").startswith("https://www.w3.org/StyleSheets/TR"):
+                    h.appendChild(self.head, el)
             # Ensure that all W3C links are https.
             for el in h.findAll("a", self):
                 href = el.get("href", "")
@@ -297,20 +305,20 @@ class Spec:
                 if text.startswith("http://www.w3.org") or text.startswith("http://lists.w3.org"):
                     el.text = "https" + text[4:]
             # Loaded from .include files
-            extensions.BSPrepTR(self)  # pylint: disable=no-member
+            extensions.BSPrepTR(self)  # type: ignore # pylint: disable=no-member
 
         return self
 
-    def serialize(self):
+    def serialize(self) -> t.Optional[str]:
         try:
             rendered = h.Serializer(self.md.opaqueElements, self.md.blockElements).serialize(self.document)
         except Exception as e:
             m.die(str(e))
-            return
+            return None
         rendered = u.finalHackyCleanup(rendered)
         return rendered
 
-    def fixMissingOutputFilename(self, outputFilename):
+    def fixMissingOutputFilename(self, outputFilename: t.Optional[str]) -> str:
         if outputFilename is None:
             # More sensible defaults!
             if isinstance(self.inputSource, InputSource.TarInputSource):
@@ -325,12 +333,12 @@ class Spec:
                 outputFilename = "-"
         return outputFilename
 
-    def finish(self, outputFilename=None, newline=None):
+    def finish(self, outputFilename: t.Optional[str] = None, newline: t.Optional[str] = None) -> None:
         catchArgparseBug(outputFilename)
         self.printResultMessage()
         outputFilename = self.fixMissingOutputFilename(outputFilename)
         rendered = self.serialize()
-        if not constants.dryRun:
+        if rendered and not constants.dryRun:
             try:
                 if outputFilename == "-":
                     sys.stdout.write(rendered)
@@ -340,7 +348,7 @@ class Spec:
             except Exception as e:
                 m.die(f"Something prevented me from saving the output document to {outputFilename}:\n{e}")
 
-    def printResultMessage(self):
+    def printResultMessage(self) -> None:
         # If I reach this point, I've succeeded, but maybe with reservations.
         fatals = m.messageCounts["fatal"]
         links = m.messageCounts["linkerror"]
@@ -357,7 +365,7 @@ class Spec:
             m.success("Successfully generated, with warnings")
             return
 
-    def watch(self, outputFilename, port=None, localhost=False):
+    def watch(self, outputFilename: t.Optional[str], port: int = None, localhost: bool = False) -> None:
         import time
 
         outputFilename = self.fixMissingOutputFilename(outputFilename)
@@ -418,7 +426,7 @@ class Spec:
         except Exception as e:
             m.die(f"Something went wrong while watching the file:\n{e}")
 
-    def fixText(self, text, moreMacros=None):
+    def fixText(self, text: str, moreMacros: t.Dict[str, str] = None) -> str:
         # Do several textual replacements that need to happen *before* the document is parsed as h.
 
         # If markdown shorthands are on, remove all `foo`s while processing,
@@ -426,6 +434,7 @@ class Spec:
         # Also handle markdown escapes.
         if moreMacros is None:
             moreMacros = {}
+        textFunctor: func.Functor
         if "markdown" in self.md.markupShorthands:
             textFunctor = u.MarkdownCodeSpans(text)
         else:
@@ -439,7 +448,7 @@ class Spec:
 
         return textFunctor.extract()
 
-    def printTargets(self):
+    def printTargets(self) -> None:
         m.p("Exported terms:")
         for el in h.findAll("[data-export]", self):
             for term in config.linkTextsFromElement(el):
@@ -449,7 +458,7 @@ class Spec:
             for term in config.linkTextsFromElement(el):
                 m.p("  " + term)
 
-    def isOpaqueElement(self, el):
+    def isOpaqueElement(self, el: t.ElementT) -> bool:
         if el.tag in self.md.opaqueElements:
             return True
         if el.get("data-opaque") is not None:
@@ -457,7 +466,7 @@ class Spec:
         return False
 
 
-def printDone():
+def printDone() -> None:
     contents = f"Finished at {datetime.now().strftime('%H:%M:%S %b-%d-%Y')}"
     contentLen = len(contents) + 2
     if not constants.asciiOnly:
@@ -472,7 +481,7 @@ def printDone():
         m.p("")
 
 
-def findImplicitInputFile():
+def findImplicitInputFile() -> t.Optional[str]:
     """
     Find what input file the user *probably* wants to use,
     by scanning the current folder.
@@ -499,7 +508,7 @@ def findImplicitInputFile():
     return None
 
 
-def catchArgparseBug(string):
+def catchArgparseBug(string: t.Optional[str]) -> bool:
     # Argparse has had a long-standing bug
     # https://bugs.python.org/issue22433
     # about spaces in the values of unknown optional arguments
@@ -517,6 +526,17 @@ def catchArgparseBug(string):
     return True
 
 
+def fetchTestSuites(dataFile: retrieve.DataFileRequester) -> t.Dict[str, testsuite.TestSuite]:
+    return {k: testsuite.TestSuite(**v) for k, v in json.loads(dataFile.fetch("test-suites.json", str=True)).items()}
+
+
+def fetchLanguages(dataFile: retrieve.DataFileRequester) -> t.Dict[str, language.Language]:
+    return {
+        k: language.Language(v["name"], v["native-name"])
+        for k, v in json.loads(dataFile.fetch("languages.json", str=True)).items()
+    }
+
+
 styleColors = """
 /* Any --*-text not paired with a --*-bg is assumed to have a transparent bg */
 :root {
