@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import re
 
 import widlparser
@@ -6,18 +8,18 @@ from . import config, h, messages as m, t
 
 
 class IDLUI:
-    def warn(self, msg):
+    def warn(self, msg: str) -> None:
         m.die(msg.rstrip())
 
-    def note(self, msg):
+    def note(self, msg: str) -> None:
         m.warn(msg.rstrip())
 
 
 class IDLSilent:
-    def warn(self, msg):
+    def warn(self, msg: str) -> None:
         pass
 
-    def note(self, msg):
+    def note(self, msg: str) -> None:
         pass
 
 
@@ -176,7 +178,7 @@ class IDLMarker(widlparser.protocols.Marker):
             )
         return (None, None)
 
-    def markup_name(self, text, construct) -> MarkupReturnT:  # pylint: disable=unused-argument
+    def markup_name(self, text: str, construct) -> MarkupReturnT:  # pylint: disable=unused-argument
         # Fires for defining names: method names, arg names, interface names, etc.
         idlType = construct.idl_type
         if idlType not in config.idlTypes:
@@ -265,7 +267,7 @@ class IDLMarker(widlparser.protocols.Marker):
     def encode(self, text: str) -> str:
         return h.escapeHTML(text)
 
-    def methodLinkingTexts(self, method: widlparser.constructs.InterfaceMember) -> t.List[str]:
+    def methodLinkingTexts(self, method: widlparser.OperationRest) -> t.List[str]:
         """
         Given a method-ish widlparser Construct,
         finds all possible linking texts.
@@ -285,18 +287,18 @@ class IDLMarker(widlparser.protocols.Marker):
         texts = []
 
         if optStart is not None:
-            prefixes = [method.name]
+            prefixes = [t.cast(str, method.name)]
             if method.name == "constructor":
-                prefixes.append(method.parent.name)
+                prefixes.append(t.cast(str, method.parent.name))
             for i in range(optStart, len(method.arguments)):
-                argText = ", ".join(arg.name for arg in method.arguments[:i])
+                argText = ", ".join(t.cast(str, arg.name) for arg in list(method.arguments)[:i])
                 for prefix in prefixes:
                     texts.append(prefix + "(" + argText + ")")
 
-        texts.append(method.normal_name)
+        texts.append(t.cast(str, method.normal_name))
         if method.name == "constructor":
-            texts.append(method.parent.name + method.normal_name[11:])
-        return reversed(texts)
+            texts.append(t.cast(str, method.parent.name) + t.cast(str, method.normal_name)[11:])
+        return list(reversed(texts))
 
 
 def markupIDL(doc: t.SpecT) -> None:
