@@ -318,7 +318,7 @@ class MetadataManager:
             macros["version"] = self.ED
         macros["annotations"] = constants.testAnnotationURL
         if doc and self.vshortname in doc.testSuites:
-            macros["testsuite"] = doc.testSuites[self.vshortname]["vshortname"]
+            macros["testsuite"] = doc.testSuites[self.vshortname].vshortname
         if self.warning and len(self.warning) >= 2:
             macros["replacedby"] = self.warning[1]
         if self.warning and len(self.warning) >= 3:
